﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Runtime.CompilerServices;

using BenchmarkDotNet.Attributes;
using BenchmarkDotNet.Running;

namespace BoxingBenchmarkCore {

	class IntegerTools {
		[MethodImpl(MethodImplOptions.AggressiveInlining)]
		public static int AddTwoInt32s(int a, int b) {
			return a + b;
		}

		[MethodImpl(MethodImplOptions.AggressiveInlining)]
		public static long AddTwoInt64s(long a, long b) {
			return a + b;
		}

		[MethodImpl(MethodImplOptions.AggressiveInlining)]
		public static object AddTwoInt32sAsObjects(object a, object b) {
			return (int) a + (int) b;
		}

		public static int AddInt32s(params int[] values) {
			int result = 0;
			for (int i = 0; i < values.Length; i++) {
				result += values[i];
			}
			return result;
		}

		public static long AddInt64s(params long[] values) {
			long result = 0;
			for (int i = 0; i < values.Length; i++) {
				result += values[i];
			}
			return result;
		}

		public static long AddIntegers(params object[] values) {
			long result = 0;
			for (int i = 0; i < values.Length; i++) {
				long longValue;
				if (values[i].GetType() == typeof(int)) {
					longValue = (int) values[i];
				} else if (values[i].GetType() == typeof(long)) {
					longValue = (long) values[i];
				} else {
					throw new ArgumentException($"Argument {i} of type {values[i].GetType()} is neither Int32 or Int64.", "values");
				}
				result += longValue;
			}
			return result;
		}
	}

	class Program {
		static void Main(string[] args) {
			BenchmarkRunner.Run<AddTests>();

			int iA = 2;
			Console.WriteLine(IntegerTools.AddInt32s(1, iA, 3));

			long lB = 1;
			Console.WriteLine(IntegerTools.AddInt64s(lB, iA, 3));

			Console.WriteLine(IntegerTools.AddIntegers(lB, iA, 3));

			byte b = 0;
			Console.WriteLine(IntegerTools.AddInt32s(1, iA, 3, b));
			Console.WriteLine(IntegerTools.AddInt64s(lB, iA, 3, b));

			try {
				Console.WriteLine(IntegerTools.AddIntegers(lB, iA, 3, b));
			} catch (Exception ex) {
				Console.WriteLine(ex);
			}
		}
	}

	public class AddTests {
		public int iA = 2;
		public long lB = 1;

		[Benchmark]
		public int AddTwoInt32sTest() {
			return IntegerTools.AddTwoInt32s(iA, iA);
		}

		[Benchmark]
		public long AddTwoInt64sTest() {
			return IntegerTools.AddTwoInt64s(lB, lB);
		}

		[Benchmark]
		public object AddTwoInt32sAsObjectsTest() {
			return IntegerTools.AddTwoInt32sAsObjects(iA, iA);
		}

		[Benchmark]
		public int AddInt32sTest() {
			return IntegerTools.AddInt32s(1, iA, 3);
		}

		[Benchmark]
		public long AddInt64sTest() {
			return IntegerTools.AddInt64s(lB, iA, 3);
		}

		[Benchmark]
		public long AddIntegersTest() {
			return IntegerTools.AddIntegers(lB, iA, 3);
		}
	}
}
